@@ -1,13 +1,16 @@
 # Local
 from .Items import SongData
-from .SymbolFixer import fix_song_name
-from .MegaMixSongData import SONG_DATA
+from .SymbolFixer import format_song_name
+from .MegaMixSongData import SONG_DATA, base_game_ids, dlc_ids
 from .DataHandler import extract_mod_data_to_json
 
 # Python
 from typing import Dict, List
 from collections import ChainMap
+import logging
 
+logging.basicConfig(level=logging.DEBUG)
+logger = logging.getLogger(__name__)
 
 class MegaMixCollections:
     """Contains all the data of MegaMix, loaded from songData.json"""
@@ -40,18 +43,53 @@ class MegaMixCollections:
 
         self.song_items = SONG_DATA
         mod_data = extract_mod_data_to_json()
-        base_game_ids = {song_data.songID for song_data in SONG_DATA.values() if song_data.songID is not None}
+
+        self.mod_remaps: dict[int, dict[str, list]] = {}
 
         if mod_data:
+            seen_mod_song_ids = set()
+            seen_mod_item_ids = set()
+
             for data_dict in mod_data:
-                for _, songs in data_dict.items():
+                for pack, songs in data_dict.items():
                     for song in songs:
+                        if not isinstance(song, list) or not list(map(type, song)) == [str, int, int]:
+                            logger.warning("Skipping", pack, song)
+                            continue
+
                         song_id = song[1]
-                        song_name = f"{fix_song_name(song[0])} [{song_id}]"
-                        item_id = (song_id * 10)
-                        # If cover song
+
                         if song_id in base_game_ids:
-                            item_id += 1
+                            continue
+
+                        song_name = format_song_name(song[0], song_id)
+                        item_id = (song_id * 10)
+
+                        if song_name in self.song_items:
+                            logger.warning(f"{song_name} previously mapped to base ID, skipping")
+                            continue
+
+                        # Remap up to 4 ID conflicts using the 8 free slots (2-9) between item/loc IDs.
+                        if song_id in seen_mod_song_ids:
+                            if song_id in self.mod_remaps and song_name in self.mod_remaps[song_id]:
+                                logger.warning(f"{song_name} already remapped to {self.mod_remaps[song_id][song_name]}")
+                                continue
+
+                            resolve = {i for i in range(item_id + 2, item_id + 10)}
+                            resolve -= seen_mod_item_ids
+                            new_slots = sorted(resolve)[0:2]
+
+                            if len(new_slots) != 2:
+                                raise Exception(f"Could not remap conflict of {song_name} (out of slots)\n"
+                                                f"{self.mod_remaps[song_id]}")
+                            logger.warning(f"Remapped {song_name} to {new_slots}")
+
+                            item_id = new_slots[0]
+                            seen_mod_item_ids.update(new_slots)
+
+                            self.mod_remaps.setdefault(song_id, {})
+                            self.mod_remaps[song_id][song_name] = new_slots
+                        seen_mod_song_ids.add(song_id)
 
                         # Shift difficulty bitfields from modded data into [#,#,#,#,#]
                         diff_info = []
@@ -62,20 +100,14 @@ class MegaMixCollections:
                             diff_info.insert(0, diff + (.5 if half else 0.0))
                             song[2] >>= 5
 
-                        self.song_items[song_name] = SongData(item_id, song_id, [], False, True, diff_info)
+                        self.song_items[song_name] = SongData(item_id, song_id, set(), song_id in dlc_ids, True, diff_info)
 
         self.item_names_to_id.update({name: data.code for name, data in self.song_items.items()})
 
         for song_name, song_data in self.song_items.items():
-            if song_data.code % 2 != 0:  # Fix code for covers
-                for i in range(2):
-                    self.song_locations[f"{song_name}-{i}"] = (song_data.code + i - 1)
-                continue
-
             for i in range(2):
                 self.song_locations[f"{song_name}-{i}"] = (song_data.code + i)
 
-
     def get_songs_with_settings(self, dlc: bool, mod_ids: List[int], allowed_diff: List[int], diff_lower: float, diff_higher: float) -> List[str]:
         """Gets a list of all songs that match the filter settings. Difficulty thresholds are inclusive."""
         filtered_list = []
